package lightJackson;

import static org.junit.Assert.assertNull;
import static org.junit.jupiter.api.Assertions.assertEquals;

import java.io.IOException;
import java.time.format.DateTimeFormatter;
import java.util.LinkedHashMap;

import org.junit.jupiter.api.Test;

import com.fasterxml.jackson.jr.annotationsupport.JacksonAnnotationExtension;
import com.fasterxml.jackson.jr.ob.JSON;
import com.fasterxml.jackson.jr.ob.JacksonJrExtension;
import com.fasterxml.jackson.jr.ob.api.ExtensionContext;

import lightJackson.data.Person;

class DeserializePersonTest {

	@Test
	void test_deserialize_json_via_annotation() {
		try {

			String json = JSON.std.with(JSON.Feature.PRETTY_PRINT_OUTPUT).asString(new LinkedHashMap<String, Object>() {
				{
					put("name", "John Doe");
					put("age", 30);
					put("ignoredElement", "shout NOT be mapped");
					put("email", "johndoe@example.com");
					put("override_name", "override Mary");
				}
			});

			Person person = JSON.builder().register(JacksonAnnotationExtension.std).build()
					.with(JSON.Feature.PRETTY_PRINT_OUTPUT).beanFrom(Person.class, json);

			assertEquals("John Doe", person.getName());
			assertEquals(30, person.getAge());
			assertEquals("johndoe@example.com", person.getEmail());
			assertEquals("override Mary", person.getOverrideName());
			assertNull(person.getIgnoredElement());
			assertNull(person.getBirthDate());
			assertNull(person.getLogTimeStamp());
		
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	@Test
	void test_deserialize_json_via_standard() {
		try {

			String json = JSON.std.with(JSON.Feature.PRETTY_PRINT_OUTPUT).asString(new LinkedHashMap<String, Object>() {
				{
					put("name", "John Doe");
					put("age", 30);
					put("email", "johndoe@example.com");
					put("ignoredElement", "should be mapped");
					put("overrideName", "override Mary");
				}
			});

			Person person = JSON.std.with(JSON.Feature.PRETTY_PRINT_OUTPUT).beanFrom(Person.class, json);

			assertEquals("John Doe", person.getName());
			assertEquals(30, person.getAge());
			assertEquals("johndoe@example.com", person.getEmail());
			assertEquals("should be mapped", person.getIgnoredElement());
			assertEquals("override Mary", person.getOverrideName());
			assertNull(person.getBirthDate());
			assertNull(person.getLogTimeStamp());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	@Test
	void test_deserialize_json_with_cust() {
		try {

			String json = JSON.std.with(JSON.Feature.PRETTY_PRINT_OUTPUT).asString(new LinkedHashMap<String, Object>() {
				{
					put("name", "John Doe");
					put("age", 30);
					put("email", "johndoe@example.com");
					put("birthDate", "1980-12-12");
					put("logTimeStamp", "2014-12-09T13:50:51.644000Z");
				}
			});

			JSON customizeMapper = JSON.builder().register(new JacksonJrExtension() {
				@Override
				protected void register(ExtensionContext extensionContext) {
					extensionContext.insertProvider(new MyHandlerProvider());
				}

			}).build().with(JSON.Feature.PRETTY_PRINT_OUTPUT, JSON.Feature.WRITE_NULL_PROPERTIES);

			Person person = customizeMapper.beanFrom(Person.class, json);

			assertEquals("John Doe", person.getName());
			assertEquals(30, person.getAge());
			assertEquals("johndoe@example.com", person.getEmail());
			assertEquals("1980-12-12", person.getBirthDate().format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
			assertEquals("2014-12-09T13:50:51.644000Z",
					person.getLogTimeStamp().format(DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss.SSSSSS'Z'")));
			assertNull(person.getIgnoredElement());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	@Test
	void test_deserialize_json_with_cust_anno() {
		try {

			String json = JSON.std.with(JSON.Feature.PRETTY_PRINT_OUTPUT).asString(new LinkedHashMap<String, Object>() {
				{
					put("name", "John Doe");
					put("age", 30);
					put("email", "johndoe@example.com");
					put("birthDate", "1980-12-12");
					put("logTimeStamp", "2014-12-09T13:50:51.644000Z");
					put("override_name", "override Mary");
				}
			});

			JSON customizeMapper = JSON.builder().register(JacksonAnnotationExtension.std)
					.register(new JacksonJrExtension() {
						@Override
						protected void register(ExtensionContext extensionContext) {
							extensionContext.insertProvider(new MyHandlerProvider());
						}

					}).build().with(JSON.Feature.PRETTY_PRINT_OUTPUT, JSON.Feature.WRITE_NULL_PROPERTIES);

			Person person = customizeMapper.beanFrom(Person.class, json);

			assertEquals("John Doe", person.getName());
			assertEquals(30, person.getAge());
			assertEquals("johndoe@example.com", person.getEmail());
			assertEquals("override Mary", person.getOverrideName());
			assertEquals("1980-12-12", person.getBirthDate().format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
			assertEquals("2014-12-09T13:50:51.644000Z",
					person.getLogTimeStamp().format(DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss.SSSSSS'Z'")));
			assertNull(person.getIgnoredElement());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

}
